﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace CreateThread
{
	internal class CreateThread
	{
		public static void Create()
		{
			// Do in new thread
			Thread t1 = new Thread(new ThreadStart(Run));
			t1.Start();

			// Do in main thread
			Run();
		}

		public static void CreateManyWay()
		{
			// Create ThreadStart delegate
			Thread t1 = new Thread(new ThreadStart(Run));
			t1.Start();

			// Infer by method and create delgate
			Thread t2 = new Thread(Run);
			t2.Start();

			// Use anonymous method
			Thread t3 = new Thread(delegate ()
			{
				Run();
			});
			t3.Start();

			// Use lambda expression
			Thread t4 = new Thread(() => Run());
			t4.Start();

			// Short
			new Thread(() => Run()).Start();
		}

		public static void CreateParameterized()
		{
			Thread t1 = new Thread(new ParameterizedThreadStart(Calc));
			t1.Start(10.0);

			Thread t2 = new Thread(() => Sum(10, 20, 30));
			t2.Start();
		}

		public static void CreateOtherClassMethod()
		{
			OuterClass obj = new OuterClass();
			Thread t = new Thread(obj.Run);
			t.Start();
		}

		public static void CreateBackgroundThread()
		{
			Thread t = new Thread(new ThreadStart(EndlessRun));
			t.IsBackground = true;
			t.Start();
		}

		public static void CreateForegroundThread()
		{
			Thread t = new Thread(new ThreadStart(EndlessRun));
			t.IsBackground = false; // default
			t.Start();
		}

		private static void Run()
		{
			Console.WriteLine($"Thread#{Thread.CurrentThread.ManagedThreadId}: Start");
			// Do something
			Thread.Sleep(3000);
			Console.WriteLine($"Thread#{Thread.CurrentThread.ManagedThreadId}: Finish");
		}

		private static void Calc(object radius)
		{
			double r = (double)radius;
			double area = r * r * Math.PI;
			Console.WriteLine($"r={radius}, area={area}");
		}

		private static void Sum(int d1, int d2, int d3)
		{
			int sum = d1 + d2 + d3;
			Console.WriteLine($"sum={sum}");
		}

		private static void EndlessRun()
		{
			int i = 0;
			while (true)
			{
				Console.WriteLine($"running... ({++i})");
				Thread.Sleep(1000);
			}
		}
	}

	internal class OuterClass
	{
		public void Run()
		{
			Console.WriteLine("OuterClass.Run");
		}
	}
}
