﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace UseBackgroundWorker
{
	public partial class Form1 : Form
	{
		private readonly int COUNT = 100;

		private BackgroundWorker worker;

		public Form1()
		{
			InitializeComponent();
		}

		private void runButton_Click(object sender, EventArgs e)
		{
			worker = new BackgroundWorker();
			worker.WorkerReportsProgress = true;
			worker.WorkerSupportsCancellation = true;

			worker.DoWork += Worker_DoWork;
			worker.ProgressChanged += Worker_ProgressChanged;
			worker.RunWorkerCompleted += Worker_RunWorkerCompleted;

			worker.RunWorkerAsync();
		}

		private void cancelButton_Click(object sender, EventArgs e)
		{
			if (worker == null)
				return;

			worker.CancelAsync();
		}

		private void Worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			// event on ui thread
			if (e.Cancelled)
				progressText.Text = "Canceled";
			else if (e.Error != null)
				progressText.Text = e.Error.ToString();
			else
				progressText.Text = $"{e.Result} files uploaded";
		}

		private void Worker_ProgressChanged(object sender, ProgressChangedEventArgs e)
		{
			// event on ui thread
			progressText.Text = $"Progress: {e.ProgressPercentage}%";
		}

		private void Worker_DoWork(object sender, DoWorkEventArgs e)
		{
			// event on worker thread
			for (int i = 0; i < COUNT; i++)
			{
				if (worker.CancellationPending)
				{
					e.Cancel = true;
					return;
				}

				string filename = $"Data_{i}.txt";
				Upload(filename);

				worker.ReportProgress(i, filename);
			}

			e.Result = COUNT;
		}

		private void Upload(string filename)
		{
			Debug.WriteLine($"Uploading {filename}");
			Thread.Sleep(1000);
		}
	}
}
